@@ -4,14 +4,15 @@ Tools for working with counts from multiple categories of data at once.
 Categories are modeled as stakced 2D arrays.    Each category is in its
 own slice of the stack.
 """
-
+from __future__ import print_function, division
+import util
 import core
 import numpy as np
 from math import log
 
 
 # ------------ Aggregators -------------------
-class CountCategories(core.Aggregator):
+class CountCategories(core.GlyphAggregator):
     """
     Count the number of items that fall into a particular grid element.
 
@@ -22,24 +23,25 @@ class CountCategories(core.Aggregator):
     call to allocate.
 
     TODO: This seems convoluted...is there a better way?
-
     """
     out_type = np.int32
     identity = np.asarray([0])
     cats = None
 
-    def allocate(self, width, height, glyphset, infos):
+    def allocate(self, glyphset, screen):
         """Allocates one array slice for each unique info passed.
            Output array shape is (#cats, height, width).
+
         """
-        self.cats = np.unique(infos)
+        (width, height) = screen
+        self.cats = np.unique(glyphset.data())
         return np.zeros((height, width, len(self.cats)), dtype=self.out_type)
 
     def combine(self, existing, points, shapecode, val):
         entry = np.zeros(len(self.cats))
         idx = np.nonzero(self.cats == val)[0][0]
         entry[idx] = 1
-        update = core.glyphAggregates(points, shapecode, entry, self.identity)
+        update = self.glyphAggregates(points, shapecode, entry, self.identity)
         existing[points[1]:points[3], points[0]:points[2], :] \
                 += update
 
@@ -58,6 +60,37 @@ class ToCounts(core.CellShader):
         dtype = (grid.dtype if dtype is None else dtype)
         return grid.sum(axis=2, dtype=dtype)
 
+class NonZeros(core.CellShader):
+    "How many non-zero categories are there?"
+    def shade(self, grid):
+        mask = grid!=0
+        nonzero = np.zeros_like(grid)
+        nonzero[mask] = 1
+        return nonzero.sum(axis=2)
+
+
+class Ratio(core.CellShader):
+    """
+    Compute the ratio of one category to the sum of all cateogires.
+    Any cell with sum of zero is propogated as a 'zero' in the sum
+     * focus: The "one cateogry"
+              Will pick the local 'max' cateogry if set to a negative
+    """
+    def __init__(self, focus=-1):
+        self.focus = focus
+
+    def shade(self, grid):
+        if self.focus < 0:
+            plane = grid.max(axis=2)
+        else:
+            plane = grid[:, :, self.focus]
+
+        sum = grid.sum(axis=2)
+        mask = (sum != 0)
+        ratio = plane/sum
+        ratio[~mask] = 0
+        return ratio 
+        
 
 class Select(core.CellShader):
     """Get the counts from just one category.
@@ -93,9 +126,9 @@ class MinPercent(core.CellShader):
     def __init__(self,
                  cutoff,
                  cat=0,
-                 above=core.Color(228, 26, 28, 255),
-                 below=core.Color(55, 126, 184, 255),
-                 background=core.Color(255, 255, 255, 0)):
+                 above=util.Color(228, 26, 28, 255),
+                 below=util.Color(55, 126, 184, 255),
+                 background=util.Color(255, 255, 255, 0)):
 
         self.cutoff = cutoff
         self.cat = cat
@@ -118,7 +151,7 @@ class MinPercent(core.CellShader):
 
 
 class HDAlpha(core.CellShader):
-    def __init__(self, colors, background=core.Color(255, 255, 255, 255),
+    def __init__(self, colors, background=util.Color(255, 255, 255, 255),
                  alphamin=0, log=False, logbase=10):
         """
         colors -- a list of colors in cateogry-order.
@@ -127,11 +160,15 @@ class HDAlpha(core.CellShader):
         logbase -- Base to use if log is true (default is 10)
         background -- Color for empty category list (default is white)
 
+        If C categories and N colors are presented:
+            * N == C: No problems
+            * N >  C: Only the first C colors are used
+            * N <  C: The last color is replicated to make N == C
+
         TODO: Change 'colors' to a dictionary of category-to-color mapping
         TODO: mask out zero-sum regions in alpha and opaque blend
         """
-        # self.colors = dict(zip(colors.keys(), map(lambda v: v.asarray(), colors.values)))
-        self.catcolors = np.array(map(lambda v: np.array(v, dtype=np.uint8), colors))
+        self.catcolors = np.array([np.array(v, dtype=np.uint8) for v in colors])
         self.background = np.array(background, dtype=np.uint8)
         self.alphamin = alphamin
         self.log = log
@@ -141,7 +178,17 @@ class HDAlpha(core.CellShader):
         sums = ToCounts.shade(grid, dtype=np.float32)
         mask = (sums != 0)
 
-        colors = HDAlpha.opaqueblend(self.catcolors, grid, sums)
+        #Ensure category count and color count match
+        cats = grid.shape[2]
+        catcolors = self.catcolors
+        if len(catcolors) > cats:
+            catcolors = catcolors[:cats]
+        elif len(catcolors) < cats:
+            other = catcolors[-1]
+            replicated = [other] * (cats-len(catcolors))
+            catcolors = np.vstack([catcolors, replicated])
+
+        colors = HDAlpha.opaqueblend(catcolors, grid, sums)
         colors[~mask] = self.background
         HDAlpha.alpha(colors, sums, mask, self.alphamin, self.log, self.logbase)
         return colors
@@ -154,7 +201,7 @@ class HDAlpha(core.CellShader):
         if (dolog):
             base = log(base)
             maxval = log(maxval)/base
-            sums[mask] = log(sums[mask])/base
+            sums[mask] = np.log10(sums[mask])/base
 
         np.putmask(colors[:, :, 3],
                    mask,
